<?php

namespace Oro\Bundle\EntityExtendBundle\Form\Type;

use Symfony\Component\Form\FormView;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;

use Oro\Bundle\EntityConfigBundle\Config\ConfigManager;
use Oro\Bundle\EntityConfigBundle\Config\Id\EntityConfigId;
use Oro\Bundle\EntityBundle\ORM\EntityClassResolver;

class AssociationChoiceType extends AbstractAssociationChoiceType
{
    /** @var EntityClassResolver */
    protected $entityClassResolver;

    /**
     * @param ConfigManager       $configManager
     * @param EntityClassResolver $entityClassResolver
     */
    public function __construct(ConfigManager $configManager, EntityClassResolver $entityClassResolver)
    {
        parent::__construct($configManager);
        $this->entityClassResolver = $entityClassResolver;
    }

    /**
     * {@inheritdoc}
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        parent::setDefaultOptions($resolver);

        $resolver->setDefaults(
            [
                'empty_value'       => false,
                'choices'           => ['No', 'Yes'],
                'association_class' => null // can be full class name or entity name
            ]
        );
    }

    /**
     * {@inheritdoc}
     */
    protected function isSchemaUpdateRequired($newVal, $oldVal)
    {
        return true == $newVal && false == $oldVal;
    }

    /**
     * {@inheritdoc}
     */
    protected function isReadOnly($options)
    {
        /** @var EntityConfigId $configId */
        $configId  = $options['config_id'];
        $className = $configId->getClassName();

        // disable for owning side entity
        if ($className === $this->entityClassResolver->getEntityClass($options['association_class'])) {
            return true;
        }

        return parent::isReadOnly($options);
    }

    /**
     * {@inheritdoc}
     */
    protected function disableView(FormView $view)
    {
        parent::disableView($view);
        $this->appendClassAttr($view->vars, 'disabled-choice');
    }

    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return 'oro_entity_extend_association_choice';
    }

    /**
     * {@inheritdoc}
     */
    public function getParent()
    {
        return 'choice';
    }
}
